//
//  TicTacToeSynchronizationServer_Tests.m
//  TicTacToe
//
//  Created by Ingo Kasprzak on 03.10.13.
//  Copyright (c) 2013 Silutions. All rights reserved.
//

@import XCTest;
#import "MultipeerConnectivitySynchronizationServer.h"

@interface DeviceDummy : NSObject
@property (strong) NSString *uid;
@end

@implementation DeviceDummy
@synthesize uid;
@end


@interface TicTacToeSynchronizationServer_Tests : XCTestCase <SynchronizationServerDelegate>
@property (strong) MultipeerConnectivitySynchronizationServer *server;
@property (strong) id device;
@end

@implementation TicTacToeSynchronizationServer_Tests

- (void)setUp {
    [super setUp];
    // Put setup code here; it will be run once, before the first test case.

	// Create dummy device object
	self.device = [[DeviceDummy alloc] init];
	[[self device] setUid:@"uid" ];
	
	self.server = [[MultipeerConnectivitySynchronizationServer alloc] init];
	self.server.delegate = self;
	[self.server startServer];
	
	// Set fake device UIDs and peer identifier strings
	[self.server addConnectedDevice:@"123" withPeerIdentifier:@"345"];
	[self.server addConnectedDevice:@"234" withPeerIdentifier:@"456"];
}

- (void)tearDown {
	[self.server stopServer];
	self.server = nil;
	
    // Put teardown code here; it will be run once, after the last test case.
    [super tearDown];
}

- (void)testConnectionManagement {
	// Test return values for different uid queries (device uids and peer ids)
	XCTAssertNotNil([self.server connectedDeviceUIDs], @"Should never be nil!");
	
	XCTAssertNil([self.server deviceUIDForPeerID:@"000"], @"Giving an unkown peerID should return nil!");
	XCTAssertNotNil([self.server deviceUIDForPeerID:@"345"], @"Giving an kown peerID should NOT return nil!");
	
	XCTAssertNotNil([self.server peerIdentifiersForDeviceUIDs:nil], @"Giving nil should not return nil!");
	XCTAssertEqual([self.server peerIdentifiersForDeviceUIDs:nil].count, (NSUInteger)0, @"Giving nil should return empty array!");

	XCTAssertNotNil([self.server peerIdentifiersForDeviceUIDs:@[@"000"]], @"Giving wrong values should return nil!");
	XCTAssertEqual([self.server peerIdentifiersForDeviceUIDs:@[@"000"]].count, (NSUInteger)0, @"Giving wrong values should return empty array!");
	
	XCTAssertEqual([self.server connectedDeviceUIDs].count, (NSUInteger)2, @"Array should have length of 2!");
}

- (void)testSendData {
	// Test sending data with broken parameters.
	NSData *data = [NSData data];
	NSArray *array = @[@"000", @"345"];
	XCTAssertFalse([self.server sendData:data toDevices:nil], @"No receivers, should return NO!");
	XCTAssertFalse([self.server sendData:data toDevices:array], @"At least 1 wrong device, should return NO!");

	XCTAssertFalse([self.server sendData:data toPeers:nil], @"No receivers, should return NO!");
	XCTAssertFalse([self.server sendData:data toPeers:array], @"At least 1 wrong peer, should return NO!");
}

- (void)testSendObjectToDevices {
	// Test sending object to device with broken parameters.
	NSArray *array = @[@"000", @"345"];
	XCTAssertFalse([self.server sendObject:@"000" withCommand:SyncDatabaseSyncIdentificationCommand toDevices:array wantsReceipt:YES],
				    @"At least 1 wrong device, should return NO!");
}

- (void)testSendObjectToPeers {
	// Test sending object to peer with broken parameters.
	NSArray *array = @[@"000", @"345"];
	XCTAssertFalse([self.server sendObject:@"000" withCommand:SyncDatabaseSyncIdentificationCommand toPeers:array wantsReceipt:YES],
				   @"At least 1 wrong peer, should return NO!");
}


#pragma mark - Synchronisation Server Delegate Methods
// Delegate methods for SynchronizationServerDelegate

- (Device*)thisDevice {
	return self.device;
}
- (void)receivedModifiedObjects:(NSSet*)objects {}
- (void)receivedBidirectionalRequestAndModifiedObjects:(NSSet*)objects {}


@end
