//
//  NSManagedObject+Synchronization.m
//  TicTacToe
//
//  Created by Kasprzak Ingo on 29.06.12.
//  Copyright (c) 2012 Silutions. All rights reserved.
//

#import "NSManagedObject+Synchronization.h"

@implementation NSManagedObject (Synchronization)

// Erzeuge ein Dictionary von den Entity-Attributen.
// Relationships werden nicht als Objekt übergeben, sondern nur der eindeutige UID-String des Objekts
- (NSDictionary *)propertyDictionary {
	// Alle Attribute der Entität holen
	NSDictionary *source = self.entity.attributesByName;

	// Attribute auslesen und in ein Dictionary schreiben
	NSMutableDictionary *destination = [NSMutableDictionary dictionaryWithCapacity:[source count]];
	[source enumerateKeysAndObjectsUsingBlock:^(id key, id obj, BOOL *stop) {
		id value = [self valueForKey:key];
		if (!value) { value = [NSNull null]; }
		[destination setObject:value forKey:key];
	}];
	
	// Alle Relationships der Entität holen
	source = self.entity.relationshipsByName;
	// Relationships als UID-Strings ins Dictionary schreiben
	[source enumerateKeysAndObjectsUsingBlock:^(id key, id obj, BOOL *stop) {
		if (![obj isToMany]) {
			id value = nil;
			if ([self valueForKey:key]) {
				value = [self valueForKeyPath:[NSString stringWithFormat:@"%@.%@", key, ManagedObjectUIDKey]];
			} else {
				value = [NSNull null];
			}
			[destination setObject:value forKey:key];
		}
	}];

	// Den Namen der Klasse dieses Objects ebenfalls ins Dictionary schreiben
	[destination setObject:NSStringFromClass([self class]) forKey:PropertyDictionaryEntityNameKey];
	return destination;
}

@end
