//
//  NSManagedObject+UID.m
//  DieBauern
//
//  Created by Kasprzak Ingo on 24.02.12.
//  Copyright (c) 2012 Silutions. All rights reserved.
//

#import "NSManagedObject+UID.h"
#import "Device+Methods.h"
#import "DeletionInfo+Methods.h"
#import "SynchronizationInfo+Methods.h"
@import ObjectiveC;

__strong NSDate *managedObjectUIDSaveDate = nil;
BOOL managedObjectSynchronizing = NO;

@implementation NSManagedObject (UID)

+ (BOOL)synchronizing {
    return managedObjectSynchronizing;
}

+ (void)setSynchronizing:(BOOL)synchronizing {
    managedObjectSynchronizing = synchronizing;
}

+ (NSString*)thisDeviceUid {
	return [[NSUserDefaults standardUserDefaults] stringForKey:ManagedObjectUIDKey];
}

+ (void)setThisDeviceUid:(NSString*)deviceUid {
	[[NSUserDefaults standardUserDefaults] setObject:deviceUid forKey:ManagedObjectUIDKey];
}

- (void)awakeFromInsert_custom {
	[self awakeFromInsert_custom];
	
	if (![NSManagedObject synchronizing]) {
		CFUUIDRef uuid = CFUUIDCreate(kCFAllocatorDefault);
		NSString* myUid = (__bridge_transfer NSString*)CFUUIDCreateString(kCFAllocatorDefault,uuid);
		[self performSelector:@selector(setUid:) withObject:myUid];
		NSDate *myDate = [NSDate date];
		[self performSelector:@selector(setCreationDate:) withObject:myDate];
		[self performSelector:@selector(setModificationDate:) withObject:myDate];
		
		NSString *deviceUid = nil;
		if ([[[self entity] managedObjectClassName] isEqualToString:NSStringFromClass([Device class])]) {
			deviceUid = myUid;
			[NSManagedObject setThisDeviceUid:myUid];
		} else {
			deviceUid = [NSManagedObject thisDeviceUid];
		}
		[self performSelector:@selector(setModificationDeviceUid:) withObject:deviceUid];
		CFRelease(uuid);
	}
}

-(void)willSave_custom {
	[self willSave_custom];

	if ((![self isInserted]) && (![self isDeleted]) && (![NSManagedObject synchronizing])) {
		if (![managedObjectUIDSaveDate isEqualToDate:[self performSelector:@selector(modificationDate)]]) {
			[self performSelector:@selector(setModificationDate:)
					   withObject:managedObjectUIDSaveDate];
			[self performSelector:@selector(setModificationDeviceUid:)
					   withObject:[NSManagedObject thisDeviceUid]];
		}
	}
}

- (void)prepareForDeletion_custom {
	[self prepareForDeletion_custom];
	if ((![NSManagedObject synchronizing]) &&
		(![self isKindOfClass:[DeletionInfo class]]) &&
		(![self isKindOfClass:[SynchronizationInfo class]])) {

		DeletionInfo *del = [NSEntityDescription insertNewObjectForEntityForName:NSStringFromClass([DeletionInfo class])
														  inManagedObjectContext:self.managedObjectContext];
		del.objectUid = [self performSelector:@selector(uid)];
		del.entityType = self.entity.managedObjectClassName;
		DBLog(@"Deletion info for deleted %@-object created.", self.entity.managedObjectClassName);
	}
}


+ (BOOL)shouldReturnYes { return NO; }
+ (BOOL)shouldReturnNo { return YES; }

@end

@implementation NSManagedObjectContext (SyncDatabaseSyncSenderUIDKey)

- (BOOL)save_custom:(NSError **)error {
	managedObjectUIDSaveDate = [NSDate date];
	return [self save_custom:error];
}

@end
