//
//  TicTacToeDatabase.m
//  TicTacToe
//
//  Created by Kasprzak Ingo on 28.06.12.
//  Copyright (c) 2012 Silutions. All rights reserved.
//

#import "TicTacToeDatabase.h"

#import "Game+Methods.h"
#import "Move+Methods.h"

@implementation TicTacToeDatabase

+ (TicTacToeDatabase*)sharedTicTacToeDatabase {
	static dispatch_once_t pred = 0;
	__strong static id _sharedObject = nil;
	dispatch_once(&pred, ^{
		// Some method swizzling on NSMAnagedObject
		_sharedObject = [[self alloc] initWithDatabaseName:@"TicTacToe"];
	});
	return _sharedObject;
}

- (Game*)activeGame { // Aktives Spiel
	Game *result = nil;
	
	NSSortDescriptor *descriptor = [NSSortDescriptor sortDescriptorWithKey:ManagedObjectCreationDateKey
																 ascending:NO];
	NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:NSStringFromClass([Game class])];
	request.sortDescriptors = @[descriptor];
	request.fetchLimit = 1;

	NSArray *fetch = [self.managedObjectContext executeFetchRequest:request error:nil];

	if ([fetch count]) {
		result = [fetch lastObject];
	} else {
		result = [self newGame];
	}
	return result;
}

- (Game*)newGame { // Neues Spiel erstellen
	Game *game = [NSEntityDescription insertNewObjectForEntityForName:NSStringFromClass([Game class])
											   inManagedObjectContext:self.managedObjectContext];
     // setting of properties for entity in category NSManagedObject+UID: - (void)awakeFromInsert_custom
	game.serverDeviceUID = [NSManagedObject thisDeviceUid]; // deviceUID des das Spiel eröffnenden Device
	return game;
}

- (Move*)newMoveForActiveGame {
	Game *game = self.activeGame;
	if (!game) { return nil; }
	
	Move *move = [NSEntityDescription insertNewObjectForEntityForName:NSStringFromClass([Move class])
											   inManagedObjectContext:self.managedObjectContext];
	move.player = @([game.moves count] % 2);
	move.game = game;

	return move;
}

- (NSArray*)movesForGame:(Game*)game { // Spielzüge, aufsteigend sortiert nach Zeit
	return [self fetchObjectsForEntityClass:[Move class]
							 sortDescriptor:@"creationDate"
							  withPredicate:@"game=%@", game];
}

@end
